==> syncart-8.x-2.105.tar.gz: syncart/src/Controller/CartController.php <==
<?php

namespace Drupal\syncart\Controller;

use Drupal\Core\Url;
use Drupal\commerce_order\Entity\Order;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_cart\CartProviderInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\syncart\Service\SynCartServiceInterface;
use Drupal\syncart\Service\SynRenderServiceInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_product\Entity\ProductInterface;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\commerce_price\Price;

/**
 * Cart Controller.
 */
class CartController extends ControllerBase {

  const CART_ROUTE = 'syncart.cart_controller_cart';

  /**
   * Current cart.
   *
   * @var \Drupal\syncart\Service\SynCartServiceInterface
   */
  protected $cart;

  /**
   * Cart Renderer.
   *
   * @var \Drupal\syncart\Service\SynRenderServiceInterface
   */
  protected $renderer;

  /**
   * Constructs a new CartController object.
   *
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   Cart provider.
   * @param \Drupal\syncart\Service\SynCartServiceInterface $syncart
   *   Syncart cart.
   * @param \Drupal\syncart\Service\SynRenderServiceInterface $syncart_renderer
   *   Syncart Renderer.
   */
  public function __construct(
    CartProviderInterface $cart_provider,
    SynCartServiceInterface $syncart,
    SynRenderServiceInterface $syncart_renderer
  ) {
    /** @var \Drupal\syncart\Service\SynCartServiceInterface $cart */
    $this->cart = new $syncart($cart_provider);
    $this->renderer = $syncart_renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_cart.cart_provider'),
      $container->get('syncart.cart'),
      $container->get('syncart.render')
    );
  }

  /**
   * Cart.
   */
  public function checkout() {
    if ($this->cart->isEmpty()) {
      return $this->redirect('syncart.cart_controller_cart');
    }
    $cart = $this->cart->load();
    $url = Url::fromRoute('commerce_checkout.form', [
      'commerce_order' => $cart->id(),
      'step' => 'order_information',
    ]);
    return new RedirectResponse($url->toString());
  }

  /**
   * Add product variation to cart.
   */
  public function addToCart(Request $request) {
    $params = Json::decode($request->getContent());
    $config = \Drupal::config('syncart.settings');
    if ($config->get('donation')) {
      $variation = \Drupal::entityTypeManager()->getStorage('commerce_product_variation')->load($params['vid']);
      $currency_code = $variation->getPrice()->getCurrencyCode();
      $product = $variation->getProduct();
      $variation = $this->getRightVariation($product, $params['donation']);
      if (empty($variation)) {
        $variation = $this->createVariation($product, $params['donation'], $currency_code);
      }
      $params['vid'] = $variation->id();
      $params['qty'] = 1;
    }
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      $this->cart->addItem($params['vid'], $params['custom'], $params['qty']);
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Add variations to cart.
   */
  public function addToCartMultiple(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vids'])) {
      foreach ($params['vids'] as $vid) {
        $this->cart->addItem($vid, 1);
      }
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Задать количество товара в корзине.
   */
  public function setProductQuantity(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      if (!empty($params['quantity']) && round($params['quantity'], 2) > 0) {
        $vid = (int) $params['vid'];
        $quantity = round($params['quantity'], 2);
        $this->cart->setOrderItemQuantity($vid, $quantity);
      }
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Задать количество товара в корзине.
   */
  public function removeFromCart(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      $this->cart->removeOrderItem($params['vid']);
    }
    return $this->redirect(self::CART_ROUTE);
  }

  /**
   * Получение текущей корзины.
   */
  public function loadCart($cid = FALSE) {
    return new JsonResponse($this->renderer->render($cid));
  }

  /**
   * {@inheritdoc}
   */
  public function repeatOrder(Order $commerce_order) {
    $this->cart->addOrderItem($commerce_order);
    return $this->redirect('syncart.cart_controller_cart');
  }

  /**
   * {@inheritdoc}
   */
  private function getRightVariation(ProductInterface $product, Int $amount) :? ProductVariationInterface {
    foreach ($product->getVariations() as $variation) {
      if ($variation->getPrice()->getNumber() == $amount) {
        return $variation;
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  private function createVariation(ProductInterface $product, Int $amount, $currency_code) :? ProductVariationInterface {
    $variation = \Drupal::entityTypeManager()->getStorage('commerce_product_variation')
      ->create([
        'type' => 'variation',
        'product_id' => $product->id(),
        'price' => new Price($amount, $currency_code),
      ]);
    $variation->save();
    return $variation;
  }

}

==> syncart-8.x-2.130.tar.gz: syncart/src/Controller/CartController.php <==
<?php

namespace Drupal\syncart\Controller;

use Drupal\Core\Url;
use Drupal\commerce_order\Entity\Order;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_cart\CartProviderInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\syncart\Service\SynCartServiceInterface;
use Drupal\syncart\Service\SynRenderServiceInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\commerce_product\Entity\ProductInterface;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\commerce_price\Price;

/**
 * Cart Controller.
 */
class CartController extends ControllerBase {

  const CART_ROUTE = 'syncart.cart_controller_cart';

  /**
   * Current cart.
   *
   * @var \Drupal\syncart\Service\SynCartServiceInterface
   */
  protected $cart;

  /**
   * Cart Renderer.
   *
   * @var \Drupal\syncart\Service\SynRenderServiceInterface
   */
  protected $renderer;

  /**
   * Constructs a new CartController object.
   *
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   Cart provider.
   * @param \Drupal\syncart\Service\SynCartServiceInterface $syncart
   *   Syncart cart.
   * @param \Drupal\syncart\Service\SynRenderServiceInterface $syncart_renderer
   *   Syncart Renderer.
   */
  public function __construct(
    CartProviderInterface $cart_provider,
    SynCartServiceInterface $syncart,
    SynRenderServiceInterface $syncart_renderer
  ) {
    /** @var \Drupal\syncart\Service\SynCartServiceInterface $cart */
    $this->cart = new $syncart($cart_provider);
    $this->renderer = $syncart_renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_cart.cart_provider'),
      $container->get('syncart.cart'),
      $container->get('syncart.render')
    );
  }

  /**
   * Cart.
   */
  public function checkout() {
    if ($this->cart->isEmpty()) {
      return $this->redirect('syncart.cart_controller_cart');
    }
    $cart = $this->cart->load();
    $url = Url::fromRoute('commerce_checkout.form', [
      'commerce_order' => $cart->id(),
      'step' => 'order_information',
    ]);
    return new RedirectResponse($url->toString());
  }

  /**
   * Add product variation to cart.
   */
  public function addToCart(Request $request) {
    $params = Json::decode($request->getContent());
    $config = \Drupal::config('syncart.settings');
    if ($config->get('donation')) {
      $variation = \Drupal::entityTypeManager()->getStorage('commerce_product_variation')->load($params['vid']);
      $currency_code = $variation->getPrice()->getCurrencyCode();
      $product = $variation->getProduct();
      $variation = $this->getRightVariation($product, $params['donation']);
      if (empty($variation)) {
        $variation = $this->createVariation($product, $params['donation'], $currency_code);
      }
      $params['vid'] = $variation->id();
      $params['qty'] = 1;
    }
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      $this->cart->addItem($params['vid'], $params['qty']);
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Add variations to cart.
   */
  public function addToCartMultiple(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vids'])) {
      foreach ($params['vids'] as $vid) {
        $this->cart->addItem($vid, 1);
      }
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Задать количество товара в корзине.
   */
  public function setProductQuantity(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      if (!empty($params['quantity']) && round($params['quantity'], 2) > 0) {
        $vid = (int) $params['vid'];
        $quantity = round($params['quantity'], 2);
        $this->cart->setOrderItemQuantity($vid, $quantity);
      }
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Удалить товар в корзине.
   */
  public function removeFromCart(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      $this->cart->removeOrderItem($params['vid']);
    }
    return $this->redirect(self::CART_ROUTE);
  }

  /**
   * Задать примечание к позиции заказа.
   */
  public function setItemNote(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['itemId'])) {
      $this->cart->setOrderItemNote($params['itemId'], $params['note']);
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Получение текущей корзины.
   */
  public function loadCart($cid = FALSE) {
    return new JsonResponse($this->renderer->render($cid));
  }

  /**
   * {@inheritdoc}
   */
  public function repeatOrder(Order $commerce_order) {
    $this->cart->addOrderItem($commerce_order);
    return $this->redirect('syncart.cart_controller_cart');
  }

  /**
   * {@inheritdoc}
   */
  private function getRightVariation(ProductInterface $product, Int $amount) :? ProductVariationInterface {
    foreach ($product->getVariations() as $variation) {
      if ($variation->getPrice()->getNumber() == $amount) {
        return $variation;
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  private function createVariation(ProductInterface $product, Int $amount, $currency_code) :? ProductVariationInterface {
    $variation = \Drupal::entityTypeManager()->getStorage('commerce_product_variation')
      ->create([
        'type' => 'variation',
        'product_id' => $product->id(),
        'price' => new Price($amount, $currency_code),
      ]);
    $variation->save();
    return $variation;
  }

}
